==> spring-boot-ext-1.3.39-sources.jar: cn/com/duibaboot/ext/autoconfigure/hazelcast/DuibaHazelcastAutoConfiguration.java <==
package cn.com.duibaboot.ext.autoconfigure.hazelcast;

import ch.qos.logback.classic.Level;
import cn.com.duiba.boot.utils.NetUtils;
import cn.com.duibaboot.ext.autoconfigure.cloud.netflix.eureka.DuibaEurekaAutoServiceRegistration;
import com.hazelcast.config.Config;
import com.hazelcast.config.DiscoveryStrategyConfig;
import com.hazelcast.core.HazelcastInstance;
import com.hazelcast.spi.properties.GroupProperty;
import com.hazelcast.spring.context.SpringManagedContext;
import com.netflix.appinfo.ApplicationInfoManager;
import com.netflix.discovery.EurekaClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.AutoConfigureBefore;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.autoconfigure.hazelcast.HazelcastAutoConfiguration;
import org.springframework.boot.autoconfigure.hazelcast.HazelcastInstanceFactory;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.Lazy;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

/**
 * 自动配置hazelcast的Config. 服务发现使用eureka.
 */
@Configuration
@Import(DuibaHazelcastPortConfiguration.class)
@ConditionalOnClass(EurekaClient.class)
@AutoConfigureBefore(HazelcastAutoConfiguration.class)
@Slf4j
@EnableConfigurationProperties(DuibaHazelcastProperties.class)
@ConditionalOnProperty(value = "eureka.client.enabled", matchIfMissing = true)
public class DuibaHazelcastAutoConfiguration {

    static{
        ((ch.qos.logback.classic.Logger)log).setLevel(Level.INFO);
    }

    public static final String SPRING_APPLICATION_CONTEXT_KEY = "applicationContext";

    @Resource
    private ApplicationContext applicationContext;

    @Resource
    private EurekaClient eurekaClient;

    //强制注入DuibaEurekaAutoServiceRegistration，在构造HazelcastInstance之前确保已经注册到eurekaServer上。
    @Resource
    private DuibaEurekaAutoServiceRegistration duibaEurekaAutoServiceRegistration;

    @Resource
    private ApplicationInfoManager applicationInfoManager;
    @Resource
    private DuibaHazelcastProperties duibaHazelcastProperties;

    /**
     * 打开 @SpringAware 的功能
     */
    @Bean
    public SpringManagedContext springManagedContext(){
        return new SpringManagedContext();
    }

    @Bean
    @Lazy
    public Config hazelcastConfig(
            DuibaHazelcastPortConfiguration duibaHazelcastPortConfiguration,
            SpringManagedContext springManagedContext
    ){
        Config config = new Config();
//        config.setInstanceName("fuck");
        config.getNetworkConfig().getJoin().getMulticastConfig().setEnabled(false);
        config.getNetworkConfig().getJoin().getTcpIpConfig().setEnabled(false);
        config.getNetworkConfig().getJoin().getAwsConfig().setEnabled(false);

        config.setManagedContext(springManagedContext);

        //获取当前节点的所有metadata信息
        Map<String, String> metadata = applicationInfoManager.getInfo().getMetadata();

        //设置当前node的属性,把当前节点的eureka metadata中所有信息都放进去
        config.getMemberAttributeConfig().getAttributes().putAll(metadata);

        //绑定spring的applicationContext，以供后续分布式线程池中使用
        config.getUserContext().put(SPRING_APPLICATION_CONTEXT_KEY, applicationContext);

        //TODO 测试一下heap会不会被占用？以及这个参数怎么调优？以及这个的兼容性怎么样？roll update是否支持？
//        MemorySize memorySize = new MemorySize(512, MemoryUnit.MEGABYTES);
//        NativeMemoryConfig nativeMemoryConfig =
//                new NativeMemoryConfig()
//                        .setAllocatorType(NativeMemoryConfig.MemoryAllocatorType.POOLED)
//                        .setSize(memorySize)
//                        .setEnabled(true)
//                        .setMinBlockSize(16)
//                        .setPageSize(1 << 20);
//        config.setNativeMemoryConfig(nativeMemoryConfig);

        //强制hazelcast使用指定端口
        config.getNetworkConfig().setPort(duibaHazelcastPortConfiguration.getHazelcastPort());
        config.getNetworkConfig().setPortAutoIncrement(false);

        config.setProperty("hazelcast.discovery.enabled", "true");
        config.setProperty("hazelcast.logging.type", "slf4j");
        config.setProperty("hazelcast.local.localAddress", NetUtils.getLocalIp());//强制hazelcast使用这个ip来监听
        config.setProperty("hazelcast.socket.server.bind.any", "false");

        config.setProperty("hazelcast.heartbeat.failuredetector.type", "deadline");
        config.setProperty("hazelcast.heartbeat.interval.seconds", "5");
        config.setProperty("hazelcast.max.no.heartbeat.seconds", "20");
        //与其他节点的连接超时时间、单位：秒
        config.setProperty(GroupProperty.SOCKET_CONNECT_TIMEOUT_SECONDS.getName(), "2");

        for(Map.Entry<String, String> entry : duibaHazelcastProperties.getProperties().entrySet()){
            config.setProperty(entry.getKey(), entry.getValue());
        }

//        config.getManagementCenterConfig().setEnabled(true).setUrl("http://localhost:8080/hazelcast-mancenter");

        Map<String, Comparable> eurekaStrategyProperties = new HashMap<>();

        EurekaOneDiscoveryStrategyFactory.setEurekaClient(eurekaClient);

        config.getNetworkConfig().getJoin().getDiscoveryConfig().addDiscoveryStrategyConfig(
                new DiscoveryStrategyConfig(EurekaOneDiscoveryStrategy.class.getName(), eurekaStrategyProperties)
        );

        return config;
    }

    @Configuration
    @ConditionalOnProperty(name={"org.springframework.boot.test.context.SpringBootTestContextBootstrapper", "duiba.test.mock.hazelcast"}, havingValue = "false", matchIfMissing = true)
    static class HazelcastInstanceConfiguration{
        //TODO hazelcast启动需耗时6-10秒，后续看看能否优化
        @Bean(destroyMethod = "shutdown")
        @Lazy
        public HazelcastInstance hazelcastInstance(Config config) {
            checkVersion();
            long start = System.currentTimeMillis();
            HazelcastInstance instance =  new HazelcastInstanceFactory(config).getHazelcastInstance();
            long cost = System.currentTimeMillis() - start;
            log.info("hazelcast is initted in {}ms, clusterState:{}, version:{}, members:{}", cost, instance.getCluster().getClusterState(), instance.getCluster().getClusterVersion(), instance.getCluster().getMembers());

            return instance;
        }

        private void checkVersion(){
            Package pkg = HazelcastInstance.class.getPackage();
            String version = (pkg != null ? pkg.getImplementationVersion() : null);
            if(version.equals("3.11") || version.startsWith("3.11.")){
            }else{
                throw new IllegalStateException("请把hazelcast版本升级到3.11.*版本, 在你的gradle文件中加入这个配置即可升级：ext['hazelcast.version']='3.11' ");
            }
        }
    }

    /**
     * 一个mock实例，啥都不干，存在的意义是为了在执行单元测试时使用，加快单测速度
     */
    @Configuration
    @ConditionalOnProperty(name={"org.springframework.boot.test.context.SpringBootTestContextBootstrapper", "duiba.test.mock.hazelcast"}, havingValue = "true")
    static class MockHazelcastInstanceConfiguration{

        @Bean
        @Lazy
        public HazelcastInstance hazelcastInstance(Config config) {
            log.info("检测到当前正在执行单元测试，mock 一个 HazelcastInstance ");

            return new MockHazelcastInstance();
        }
    }

    //use static to break cycle reference
    @Bean
    public static HazelcastEndpoint hazelcastEndpoint() {
        return new HazelcastEndpoint();
    }

}

==> spring-boot-ext-1.3.52-sources.jar: cn/com/duibaboot/ext/autoconfigure/hazelcast/DuibaHazelcastAutoConfiguration.java <==
package cn.com.duibaboot.ext.autoconfigure.hazelcast;

import ch.qos.logback.classic.Level;
import cn.com.duiba.boot.utils.NetUtils;
import cn.com.duibaboot.ext.autoconfigure.cloud.netflix.eureka.DuibaEurekaAutoServiceRegistration;
import com.hazelcast.config.Config;
import com.hazelcast.config.DiscoveryStrategyConfig;
import com.hazelcast.core.HazelcastInstance;
import com.hazelcast.spi.properties.GroupProperty;
import com.hazelcast.spring.context.SpringManagedContext;
import com.netflix.appinfo.ApplicationInfoManager;
import com.netflix.discovery.EurekaClient;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigureBefore;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.autoconfigure.hazelcast.HazelcastAutoConfiguration;
import org.springframework.boot.autoconfigure.hazelcast.HazelcastInstanceFactory;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.Lazy;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 自动配置hazelcast的Config. 服务发现使用eureka.
 */
@Configuration
@Import(DuibaHazelcastPortConfiguration.class)
@ConditionalOnClass(EurekaClient.class)
@AutoConfigureBefore(HazelcastAutoConfiguration.class)
@Slf4j
@EnableConfigurationProperties(DuibaHazelcastProperties.class)
@ConditionalOnProperty(value = "eureka.client.enabled", matchIfMissing = true)
public class DuibaHazelcastAutoConfiguration{

    static{
        ((ch.qos.logback.classic.Logger)log).setLevel(Level.INFO);
    }

    public static final String SPRING_APPLICATION_CONTEXT_KEY = "applicationContext";

    @Resource
    private ApplicationContext applicationContext;

    @Resource
    private EurekaClient eurekaClient;

    //强制注入DuibaEurekaAutoServiceRegistration，在构造HazelcastInstance之前确保已经注册到eurekaServer上。
    @Resource
    private DuibaEurekaAutoServiceRegistration duibaEurekaAutoServiceRegistration;

    @Resource
    private ApplicationInfoManager applicationInfoManager;
    @Resource
    private DuibaHazelcastProperties duibaHazelcastProperties;

    /**
     * 打开 @SpringAware 的功能
     */
    @Bean
    public SpringManagedContext springManagedContext(){
        return new SpringManagedContext();
    }

    @Bean
    @Lazy
    public Config hazelcastConfig(
            DuibaHazelcastPortConfiguration duibaHazelcastPortConfiguration,
            SpringManagedContext springManagedContext,
            @Autowired(required = false) List<HazelcastConfigPostProcessor> hazelcastConfigPostProcessorList
    ){

        Config config = new Config();
//        config.setInstanceName("fuck");
        config.getNetworkConfig().getJoin().getMulticastConfig().setEnabled(false);
        config.getNetworkConfig().getJoin().getTcpIpConfig().setEnabled(false);
        config.getNetworkConfig().getJoin().getAwsConfig().setEnabled(false);

        config.setManagedContext(springManagedContext);

        //获取当前节点的所有metadata信息
        Map<String, String> metadata = applicationInfoManager.getInfo().getMetadata();

        //设置当前node的属性,把当前节点的eureka metadata中所有信息都放进去
        config.getMemberAttributeConfig().getAttributes().putAll(metadata);

        //绑定spring的applicationContext，以供后续分布式线程池中使用
        config.getUserContext().put(SPRING_APPLICATION_CONTEXT_KEY, applicationContext);

        //TODO 测试一下heap会不会被占用？以及这个参数怎么调优？以及这个的兼容性怎么样？roll update是否支持？
//        MemorySize memorySize = new MemorySize(512, MemoryUnit.MEGABYTES);
//        NativeMemoryConfig nativeMemoryConfig =
//                new NativeMemoryConfig()
//                        .setAllocatorType(NativeMemoryConfig.MemoryAllocatorType.POOLED)
//                        .setSize(memorySize)
//                        .setEnabled(true)
//                        .setMinBlockSize(16)
//                        .setPageSize(1 << 20);
//        config.setNativeMemoryConfig(nativeMemoryConfig);

        //强制hazelcast使用指定端口
        config.getNetworkConfig().setPort(duibaHazelcastPortConfiguration.getHazelcastPort());
        config.getNetworkConfig().setPortAutoIncrement(false);

        config.setProperty("hazelcast.discovery.enabled", "true");
        config.setProperty("hazelcast.logging.type", "slf4j");
        config.setProperty("hazelcast.local.localAddress", NetUtils.getLocalIp());//强制hazelcast使用这个ip来监听
        config.setProperty("hazelcast.socket.server.bind.any", "false");

        config.setProperty("hazelcast.heartbeat.failuredetector.type", "deadline");
        config.setProperty("hazelcast.heartbeat.interval.seconds", "5");
        config.setProperty("hazelcast.max.no.heartbeat.seconds", "20");
        //与其他节点的连接超时时间、单位：秒
        config.setProperty(GroupProperty.SOCKET_CONNECT_TIMEOUT_SECONDS.getName(), "2");

        for(Map.Entry<String, String> entry : duibaHazelcastProperties.getProperties().entrySet()){
            config.setProperty(entry.getKey(), entry.getValue());
        }

        Map<String, Comparable> eurekaStrategyProperties = new HashMap<>();

        EurekaOneDiscoveryStrategyFactory.setEurekaClient(eurekaClient);

        config.getNetworkConfig().getJoin().getDiscoveryConfig().addDiscoveryStrategyConfig(
                new DiscoveryStrategyConfig(EurekaOneDiscoveryStrategy.class.getName(), eurekaStrategyProperties)
        );

        if(Objects.nonNull(hazelcastConfigPostProcessorList)){
            postProcessorHazelcastConfig(config,hazelcastConfigPostProcessorList);

        }
        return config;
    }

    private void postProcessorHazelcastConfig(Config config,List<HazelcastConfigPostProcessor> hazelcastConfigPostProcessorList){
        for(HazelcastConfigPostProcessor postProcessor:hazelcastConfigPostProcessorList){
            try{
                postProcessor.preProcessor(config);
            }catch (Exception e){
                log.error("预处理HazelcastConfig异常",e);
                throw new RuntimeException(e);
            }
        }
    }

    @Configuration
    @ConditionalOnProperty(name={"org.springframework.boot.test.context.SpringBootTestContextBootstrapper", "duiba.test.mock.hazelcast"}, havingValue = "false", matchIfMissing = true)
    static class HazelcastInstanceConfiguration{
        //TODO hazelcast启动需耗时6-10秒，后续看看能否优化
        @Bean(destroyMethod = "shutdown")
        @Lazy
        public HazelcastInstance hazelcastInstance(Config config) {
            checkVersion();
            long start = System.currentTimeMillis();
            HazelcastInstance instance =  new HazelcastInstanceFactory(config).getHazelcastInstance();
            long cost = System.currentTimeMillis() - start;
            log.info("hazelcast is initted in {}ms, clusterState:{}, version:{}, members:{}", cost, instance.getCluster().getClusterState(), instance.getCluster().getClusterVersion(), instance.getCluster().getMembers());

            return instance;
        }

        private void checkVersion(){
            Package pkg = HazelcastInstance.class.getPackage();
            String version = (pkg != null ? pkg.getImplementationVersion() : null);
            if(!StringUtils.equals(version,"3.11") && !StringUtils.startsWith(version,"3.11.")){
                throw new IllegalStateException("请把hazelcast版本升级到3.11.*版本, 在你的gradle文件中加入这个配置即可升级：ext['hazelcast.version']='3.11' ");
            }
        }
    }


    /**
     * 一个mock实例，啥都不干，存在的意义是为了在执行单元测试时使用，加快单测速度
     */
    @Configuration
    @ConditionalOnProperty(name={"org.springframework.boot.test.context.SpringBootTestContextBootstrapper", "duiba.test.mock.hazelcast"}, havingValue = "true")
    static class MockHazelcastInstanceConfiguration{

        @Bean
        @Lazy
        public HazelcastInstance hazelcastInstance() {
            log.info("检测到当前正在执行单元测试，mock 一个 HazelcastInstance ");

            return new MockHazelcastInstance();
        }
    }

    //use static to break cycle reference
    @Bean
    public static HazelcastEndpoint hazelcastEndpoint() {
        return new HazelcastEndpoint();
    }

}
